@@ -7,5 +7,5 @@
  * file may be redistributed under the GNU Public License v2.
  */
 
-#define E2FSPROGS_VERSION "1.43.4"
-#define E2FSPROGS_DATE "31-Jan-2017"
+#define E2FSPROGS_VERSION "1.43.6"
+#define E2FSPROGS_DATE "29-Aug-2017"
